/*
 *   -- micMAGMA (version 1.2.0) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      April 2012
 *
 *   @author Azzam Haidar
 *   @author Khairul Kabir
 *   @author Mark Gates
 *
 *   @generated from magmablas_z.cpp normal z -> c, Sat May 31 21:27:53 2014
 */

#include <stdio.h>
#include <stdlib.h>
#include "magma.h"

#ifdef HAVE_MIC

#include "magma_mic_bindings.h"


// ========================================
// copying sub-matrices (contiguous columns)
extern "C" magma_int_t
magma_csetmatrix(
    magma_int_t m, magma_int_t n,
    magmaFloatComplex const* hA_src, size_t hA_offset, magma_int_t ldha,
    magmaFloatComplex_ptr    dA_dst, size_t dA_offset, magma_int_t ldda,
    magma_queue_t queue )
{
    magma_event_t event;
    magma_mic_synchronize_kernels();
    magma_mic_memcpy2D_async(
        dA_dst + dA_offset, ldda*sizeof(magmaFloatComplex),
        hA_src + hA_offset, ldha*sizeof(magmaFloatComplex),
        m*sizeof(magmaFloatComplex), n,
        WRITETO_DEV, queue, &event );
    magma_mic_fence_wait( event );
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_cgetmatrix(
    magma_int_t m, magma_int_t n,
    magmaFloatComplex_const_ptr dA_src, size_t dA_offset, magma_int_t ldda,
    magmaFloatComplex*          hA_dst, size_t hA_offset, magma_int_t ldha,
    magma_queue_t queue )
{
    magma_event_t event;
    magma_mic_synchronize_kernels();
    magma_mic_memcpy2D_async(
        hA_dst + hA_offset, ldha*sizeof(magmaFloatComplex),
        dA_src + dA_offset, ldda*sizeof(magmaFloatComplex),
        m*sizeof(magmaFloatComplex), n,
        READFROM_DEV, queue, &event );
    magma_mic_fence_wait( event );
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_csetmatrix_async(
    magma_int_t m, magma_int_t n,
    magmaFloatComplex const* hA_src, size_t hA_offset, magma_int_t ldha,
    magmaFloatComplex_ptr    dA_dst, size_t dA_offset, magma_int_t ldda,
    magma_queue_t queue, magma_event_t *event )
{
    magma_mic_memcpy2D_async(
        dA_dst + dA_offset, ldda*sizeof(magmaFloatComplex),
        hA_src + hA_offset, ldha*sizeof(magmaFloatComplex),
        m*sizeof(magmaFloatComplex), n,
        WRITETO_DEV, queue, event );
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_cgetmatrix_async(
    magma_int_t m, magma_int_t n,
    magmaFloatComplex_const_ptr dA_src, size_t dA_offset, magma_int_t ldda,
    magmaFloatComplex*          hA_dst, size_t hA_offset, magma_int_t ldha,
    magma_queue_t queue, magma_event_t *event )
{
    magma_mic_memcpy2D_async(
        hA_dst + hA_offset, ldha*sizeof(magmaFloatComplex),
        dA_src + dA_offset, ldda*sizeof(magmaFloatComplex),
        m*sizeof(magmaFloatComplex), n,
        READFROM_DEV, queue, event );
    return MAGMA_SUCCESS;
}

// --------------------
// TODO this doesn't match naming scheme; use magma_isetvector and magma_igetvector
extern "C" magma_int_t
magma_csetintarray_async(
    magma_int_t lda, magma_int_t const* hA_src, size_t hA_offset,
    magma_int_t *dA_dst, size_t dA_offset, magma_queue_t queue, magma_event_t *event )
{
    magma_int_t ncol = 1;
    magma_mic_memcpy2D_async(
        (void *)(dA_dst + dA_offset), lda * sizeof(magma_int_t),
        (void *)(hA_src + hA_offset), lda * sizeof(magma_int_t),
        lda * sizeof(magma_int_t), ncol,
        WRITETO_DEV, queue, event);
    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_cgetintarray_async(
    magma_int_t lda, magma_int_t const* hA_src, size_t hA_offset,
    magma_int_t *dA_dst, size_t dA_offset, magma_queue_t queue, magma_event_t *event )
{
    magma_mic_memcpy2D_async(
        (void *)(dA_dst + dA_offset), lda * sizeof(magma_int_t),
        (void *)(hA_src + hA_offset), lda * sizeof(magma_int_t),
        lda * sizeof(magma_int_t), 1,
        READFROM_DEV, queue, event);
    return MAGMA_SUCCESS;
}




// ========================================
// BLAS functions
// --------------------
extern "C" magma_int_t
magma_ccopy(
    magma_int_t n,
    magmaFloatComplex_ptr dx, size_t dx_offset, magma_int_t incx,
    magmaFloatComplex_ptr dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_ccopy_param param;

    param.n      = n;
    param.x      = dx + dx_offset;
    param.incx   = incx;
    param.y      = dy + dy_offset;
    param.incy    = incy;

    int control_msg = magma_mic_CCOPY;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_cgemm(
    magma_trans_t transA, magma_trans_t transB,
    magma_int_t m, magma_int_t n, magma_int_t k,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    magmaFloatComplex beta,  magmaFloatComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t handle )
{
    int err;
    magma_mic_cgemm_param param;

    param.transa = transA;
    param.transb = transB;
    param.m      = m;
    param.n      = n;
    param.k      = k;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;
    param.beta   = beta;
    param.c      = dC + dC_offset;
    param.ldc    = ldc;

    int control_msg = magma_mic_CGEMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_cgemv(
    magma_trans_t transA,
    magma_int_t m, magma_int_t n,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dx, size_t dx_offset, magma_int_t incx,
    magmaFloatComplex beta,  magmaFloatComplex_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_cgemv_param param;

    param.transa = transA;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.x      = dx + dx_offset;
    param.incx   = incx;
    param.beta   = beta;
    param.y      = dy + dy_offset;
    param.incy    = incy;

    int control_msg = magma_mic_CGEMV;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_chemm(
    magma_side_t side, magma_uplo_t uplo,
    magma_int_t m, magma_int_t n,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    magmaFloatComplex beta,  magmaFloatComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t handle )
{
    int err;
    magma_mic_chemm_param param;

    param.side   = side;
    param.uplo   = uplo;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;
    param.beta   = beta;
    param.c      = dC + dC_offset;
    param.ldc    = ldc;
    param.agboff = 0;

    int control_msg = magma_mic_CHEMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}
// --------------------
extern "C" magma_int_t
magma_chemm_ours(
    magma_side_t side, magma_uplo_t uplo,
    magma_int_t m, magma_int_t n,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    magmaFloatComplex beta,  magmaFloatComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_int_t globaldAoffset,
    magma_queue_t handle )
{
    int err;
    magma_mic_chemm_param param;

    param.side   = side;
    param.uplo   = uplo;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;
    param.beta   = beta;
    param.c      = dC + dC_offset;
    param.ldc    = ldc;
    param.ldc    = ldc;
    param.agboff = globaldAoffset;

    int control_msg = magma_mic_CHEMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_csymv2(
    magma_uplo_t uplo,
    magma_int_t n,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dx, size_t dx_offset, magma_int_t incx,
    magmaFloatComplex beta,  magmaFloatComplex_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_csymv2_param param;

    param.uplo   = uplo;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.x      = dx + dx_offset;
    param.incx   = incx;
    param.beta   = beta;
    param.y      = dy + dy_offset;
    param.incy   = incy;

    int control_msg = magma_mic_CSYMV2;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_chemv(
    magma_uplo_t uplo,
    magma_int_t n,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dx, size_t dx_offset, magma_int_t incx,
    magmaFloatComplex beta,  magmaFloatComplex_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_chemv_param param;

    param.uplo   = uplo;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.x      = dx + dx_offset;
    param.incx   = incx;
    param.beta   = beta;
    param.y      = dy + dy_offset;
    param.incy    = incy;

    int control_msg = magma_mic_CHEMV;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_cherk(
    magma_uplo_t uplo,
    magma_trans_t trans,
    magma_int_t n, magma_int_t k,
    float alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
    float beta,  magmaFloatComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t handle )
{
    int err;
    magma_mic_cherk_param param;

    param.uplo  = uplo;
    param.trans = trans;
    param.n     = n;
    param.k     = k;
    param.alpha = alpha;
    param.a     = dA + dA_offset;
    param.lda   = lda;
    param.beta  = beta;
    param.c     = dC + dC_offset;
    param.ldc   = ldc;

    int control_msg = magma_mic_CHERK;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}


// --------------------
extern "C" magma_int_t
magma_cherk_mmic(
    magma_uplo_t uplo, magma_trans_t trans,
    magma_int_t n, magma_int_t k,
    float alpha, magmaFloatComplex_const_ptr dA, magma_int_t lda, magma_int_t dAoff,
    float beta,  magmaFloatComplex_const_ptr dC, magma_int_t ldc, magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_int_t my_id )
{
    int err;
    magma_mic_cherk_mmic_param param;

    param.uplo   = uplo;
    param.trans  = trans;
    param.n      = n;
    param.k      = k;
    param.alpha  = alpha;
    param.a      = dA;
    param.lda    = lda;
    param.aoff   = dAoff;
    param.beta   = beta;
    param.c      = dC;
    param.ldc    = ldc;
    param.offset = offset;
    param.nb     = nb;
    param.nmic   = nmic;
    param.my_id  = my_id;

    int control_msg = magma_mic_CHERK_MMIC;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}



// --------------------
extern "C" magma_int_t
magma_cher2k(
    magma_uplo_t uplo, magma_trans_t trans,
    magma_int_t n, magma_int_t k,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    float              beta, magmaFloatComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t queue )
{
    int err;
    magma_mic_cher2k_param param;

    param.uplo  = uplo;
    param.trans = trans;
    param.n     = n;
    param.k     = k;
    param.alpha = alpha;
    param.a     = dA + dA_offset;
    param.lda   = lda;
    param.b     = dB + dB_offset;
    param.ldb   = ldb;
    param.beta  = beta;
    param.c     = dC + dC_offset;
    param.ldc   = ldc;

    int control_msg = magma_mic_CHER2K;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_cher2k_mmic(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, magma_int_t lda, magma_int_t dAoff,
                              magmaFloatComplex_const_ptr dB, magma_int_t ldb, magma_int_t dBoff,
    float beta,              magmaFloatComplex_const_ptr dC, magma_int_t ldc,  magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_int_t my_id)
{
    int err;
    magma_mic_cher2k_mmic_param param;

    param.uplo   = uplo;
    param.trans  = trans;
    param.n      = n;
    param.k      = k;
    param.alpha  = alpha;
    param.a      = dA;
    param.lda    = lda;
    param.aoff   = dAoff;
    param.b      = dB;
    param.ldb    = ldb;
    param.boff   = dBoff;
    param.beta   = beta;
    param.c      = dC;
    param.ldc    = ldc;
    param.offset = offset;
    param.nb     = nb;
    param.nmic   = nmic;
    param.my_id  = my_id;

    int control_msg = magma_mic_CHER2K_MMIC;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}


// --------------------
extern "C" magma_int_t
magma_ctrsm(
    magma_side_t  side,
    magma_uplo_t  uplo,
    magma_trans_t trans,
    magma_diag_t  diag,
    magma_int_t m, magma_int_t n,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_ptr       dB, size_t dB_offset, magma_int_t ldb,
    magma_queue_t handle )
{
    int err;
    magma_mic_ctrsm_param param;

    param.side   = side;
    param.uplo   = uplo;
    param.transa = trans;
    param.diag   = diag;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;

    int control_msg = magma_mic_CTRSM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_ctrsv(magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t n,
    magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
    magmaFloatComplex_ptr dx, size_t dx_offset, magma_int_t incx,
    magma_queue_t queue )
{
    int err;
    magma_mic_ctrsv_param param;
    
    param.uplo   = uplo;
    param.trans  = trans;
    param.diag   = diag;
    param.n      = n;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.dx     = dx + dx_offset;
    param.incx   = incx;
    
    int control_msg = magma_mic_CTRSV;
    
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    
    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_ctrmm(
    magma_side_t side, magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t m, magma_int_t n,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_ptr       dB, size_t dB_offset, magma_int_t ldb,
    magma_queue_t queue )
{
    int err;
    magma_mic_ctrmm_param param;

    param.side   = side;
    param.uplo   = uplo;
    param.transa = trans;
    param.diag   = diag;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;

    int control_msg = magma_mic_CTRMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_ctrmv(magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t n,
    magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
    magmaFloatComplex_ptr dx, size_t dx_offset, magma_int_t incx,
    magma_queue_t queue )
{
    int err;
    magma_mic_ctrmv_param param;
    
    param.uplo   = uplo;
    param.trans  = trans;
    param.diag   = diag;
    param.n      = n;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.dx     = dx + dx_offset;
    param.incx   = incx;
    
    int control_msg = magma_mic_CTRMV;
    
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    
    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    
    return MAGMA_SUCCESS;
}




// ========================================
// Auxuliary routines
extern "C" magma_int_t
magma_ctranspose(magmaFloatComplex_ptr odata, magma_int_t offo, magma_int_t ldo,
    magmaFloatComplex_ptr idata, magma_int_t offi, magma_int_t ldi,
    magma_int_t m, magma_int_t n,
    magma_queue_t queue)
{
    int err;
    magma_mic_ctranspose_param param;

    param.odata = odata + offo;
    param.ldo   = ldo;
    param.idata = idata + offi;
    param.ldi   = ldi;
    param.m     = m;
    param.n     = n;

    int control_msg = magma_mic_CTRANSPOSE;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_claswp(
    magma_int_t n, magmaFloatComplex_ptr d_A, magma_int_t lda,
    magma_int_t k1, magma_int_t k2, magma_int_t *d_ipiv, magma_int_t incx,
    magma_queue_t queue)
{
    magma_event_t event;
    int err;
    magma_mic_claswp_param param;
    param.n      = n;
    param.d_A    = d_A;
    param.lda    = lda;
    param.k1     = k1;
    param.k2     = k2;
    param.d_ipiv = d_ipiv;
    param.incx   = incx;

    int control_msg = magma_mic_CLASWP;
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_cswap(
    magma_int_t n, magmaFloatComplex *dA1T, magma_int_t lda1,
    magmaFloatComplex *dA2T, magma_int_t lda2,
    magma_queue_t queue)
{
    magma_event_t event;
    int err;
    magma_mic_cswap_param param;
    param.n      = n;
    param.dA1T    = dA1T;
    param.lda1    = lda1;
    param.dA2T    = dA2T;
    param.lda2    = lda2;

    int control_msg = magma_mic_CSWAP;
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    return MAGMA_SUCCESS;
}



// This version sends a larfb request to the MIC. The difference with the version in
// 'src' is that here there is only one request and control is passed immediately back
// to the CPU.
extern "C" magma_int_t
magma_clarfb(
    magma_side_t side, magma_trans_t trans, magma_direct_t direct, magma_storev_t storev,
    magma_int_t m, magma_int_t n, magma_int_t k,
    magmaFloatComplex_ptr dV, size_t dV_offset, magma_int_t ldv,
    magmaFloatComplex_ptr dT, size_t dT_offset, magma_int_t ldt,
    magmaFloatComplex_ptr dC, size_t dC_offset, magma_int_t ldc,
    magmaFloatComplex_ptr dwork, size_t dwork_offset, magma_int_t ldwork,
    magma_queue_t queue)
{
    int err;
    magma_mic_clarfb_param param;

    param.side   = side;
    param.trans  = trans;
    param.direct = direct;
    param.storev = storev;
    param.m      = m;
    param.n      = n;
    param.k      = k;
    param.v      = dV + dV_offset;
    param.ldv    = ldv;
    param.t      = dT + dT_offset;
    param.ldt    = ldt;
    param.c      = dC + dC_offset;
    param.ldc    = ldc;
    param.work   = dwork + dwork_offset;
    param.ldwork = ldwork;

    int control_msg = magma_mic_CLARFB;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_clarfb_gemm(
    magma_side_t side, magma_trans_t trans, magma_direct_t direct, magma_storev_t storev,
    magma_int_t m, magma_int_t n, magma_int_t k,
    magmaFloatComplex_ptr dV, size_t dV_offset, magma_int_t ldv,
    magmaFloatComplex_ptr dT, size_t dT_offset, magma_int_t ldt,
    magmaFloatComplex_ptr dC, size_t dC_offset, magma_int_t ldc,
    magmaFloatComplex_ptr dwork, size_t dwork_offset, magma_int_t ldwork,
    magmaFloatComplex_ptr dworkvt, size_t dworkvt_offset, magma_int_t ldworkvt,
    magma_queue_t queue)
{
    int err;
    magma_mic_clarfb_gemm_param larfb_gemm_param;

    larfb_gemm_param.side   = side;
    larfb_gemm_param.trans  = trans;
    larfb_gemm_param.direct = direct;
    larfb_gemm_param.storev = storev;
    larfb_gemm_param.m      = m;
    larfb_gemm_param.n      = n;
    larfb_gemm_param.k      = k;
    larfb_gemm_param.v      = dV + dV_offset;
    larfb_gemm_param.ldv    = ldv;
    larfb_gemm_param.t      = dT + dT_offset;
    larfb_gemm_param.ldt    = ldt;
    larfb_gemm_param.c      = dC + dC_offset;
    larfb_gemm_param.ldc    = ldc;
    larfb_gemm_param.work   = dwork + dwork_offset;
    larfb_gemm_param.ldwork = ldwork;
    larfb_gemm_param.workvt   = dworkvt + dworkvt_offset;
    larfb_gemm_param.ldworkvt = ldworkvt;

    int control_msg = magma_mic_CLARFB_GEMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &larfb_gemm_param, sizeof(larfb_gemm_param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_claset(magma_uplo_t uplo,
             magma_int_t m, magma_int_t n,
             magmaFloatComplex alpha, magmaFloatComplex beta,
             magmaFloatComplex_ptr dA, size_t dA_offset, magma_int_t lda,
             magma_queue_t queue)
{
     int err;
     magma_mic_claset_param param;
     int control_msg = magma_mic_CLASET;

     param.uplo   = uplo;
     param.m      = m;
     param.n      = n;
     param.alpha  = alpha;
     param.beta   = beta;
     param.dA     = dA + dA_offset;
     param.lda    = lda;

     if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
         err = errno;
         printf("scif_send failed with err %d\n", errno);
         fflush(stdout);
     }

     if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
     }

     return MAGMA_SUCCESS;
}


// ========================================
// BLAS functions - combined blas for MIC
extern "C" magma_int_t
magma_ctrsmgemm(
    magma_side_t side, magma_uplo_t uplo,
    magma_trans_t trans, magma_diag_t diag,
    magma_int_t tm, magma_int_t tn,
    magmaFloatComplex talpha, magmaFloatComplex_const_ptr tdA, size_t tdA_offset, magma_int_t tlda,
                               magmaFloatComplex_ptr       tdB, size_t tdB_offset, magma_int_t tldb,
    
    magma_trans_t transA, magma_trans_t transB,
    magma_int_t m, magma_int_t n, magma_int_t k,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    magmaFloatComplex beta,  magmaFloatComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t handle )
{
    int err;
    magma_mic_ctrsmgemm_param param;

    //trsm parameter
    param.trsm_param.side   = side;
    param.trsm_param.uplo   = uplo;
    param.trsm_param.transa = trans;
    param.trsm_param.diag   = diag;
    param.trsm_param.m      = tm;
    param.trsm_param.n      = tn;
    param.trsm_param.alpha  = talpha;
    param.trsm_param.a      = tdA + tdA_offset;
    param.trsm_param.lda    = tlda;
    param.trsm_param.b      = tdB + tdB_offset;
    param.trsm_param.ldb    = tldb;

    //gemm parameter
    param.gemm_param.transa = transA;
    param.gemm_param.transb = transB;
    param.gemm_param.m      = m;
    param.gemm_param.n      = n;
    param.gemm_param.k      = k;
    param.gemm_param.alpha  = alpha;
    param.gemm_param.a      = dA + dA_offset;
    param.gemm_param.lda    = lda;
    param.gemm_param.b      = dB + dB_offset;
    param.gemm_param.ldb    = ldb;
    param.gemm_param.beta   = beta;
    param.gemm_param.c      = dC + dC_offset;
    param.gemm_param.ldc    = ldc;

    int control_msg = magma_mic_CTRSM_GEMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

#endif // HAVE_MIC
