/*
 *   -- micMAGMA (version 1.2.0) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      April 2012
 *
 * @generated from magmablas_zc.cpp mixed zc -> ds, Sat May 31 21:27:53 2014
 */

// defines stub functions, if no GPU implementation is available.
#include <stdio.h>
#include <stdlib.h>
#include "magma.h"

#ifdef HAVE_MIC

#include "magma_mic_bindings.h"


extern "C" magma_int_t
magma_dlag2s( magma_int_t m, magma_int_t n,
              magmaDouble_ptr dA, size_t dA_offset, magma_int_t lda,
              magmaFloat_ptr dAs, size_t dAs_offset, magma_int_t ldas,
              magma_queue_t queue)
{
  magma_event_t event;
  int err;
  magma_mic_dlag2s_param dlag2s_param;

  dlag2s_param.m                  =       m;
  dlag2s_param.n                  =       n;
  dlag2s_param.dA                 =       dA + dA_offset;
  dlag2s_param.lda                =       lda;
  dlag2s_param.dAs                =       dAs + dAs_offset;
  dlag2s_param.ldas               =       ldas;

  int control_msg         =       magma_mic_DLAG2S;
  if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
    err = errno;
    printf("scif_send failed with err %d\n", errno);
    fflush(stdout);
  }

  if ((err = scif_send(gEpd, &dlag2s_param, sizeof(dlag2s_param), 1)) <= 0) {
    err = errno;
    printf("scif_send failed with err %d\n", errno);
    fflush(stdout);
  }
  return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_slag2d( magma_int_t m, magma_int_t n,
              magmaFloat_ptr dAs, size_t dAs_offset, magma_int_t ldas,
              magmaDouble_ptr dA, size_t dA_offset, magma_int_t lda,
              magma_queue_t queue)
{
  magma_event_t event;
  int err;
  magma_mic_slag2d_param slag2d_param;

  slag2d_param.m                  =       m;
  slag2d_param.n                  =       n;
  slag2d_param.dAs                =       dAs + dAs_offset;
  slag2d_param.ldas               =       ldas;
  slag2d_param.dA                 =       dA + dA_offset;
  slag2d_param.lda                =       lda;

  int control_msg         =       magma_mic_SLAG2D;
  if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
    err = errno;
    printf("scif_send failed with err %d\n", errno);
    fflush(stdout);
  }

  if ((err = scif_send(gEpd, &slag2d_param, sizeof(slag2d_param), 1)) <= 0) {
    err = errno;
    printf("scif_send failed with err %d\n", errno);
    fflush(stdout);
  }
  return MAGMA_SUCCESS;
}


extern "C" magma_int_t
magma_dsgesv(magma_int_t N, magma_int_t NRHS,
             magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
             magma_int_t *dIPIV,
             magmaDouble_ptr dB, size_t dB_offset, magma_int_t lddb,
             magmaDouble_ptr dX, size_t dX_offset, magma_int_t lddx,
             magmaDouble_ptr dworkd, magmaFloat_ptr dworks,
             magma_queue_t queue )
{
  magma_event_t event;
  int err;
  magma_mic_dsgesv_param dsgesv_param;

  dsgesv_param.n      = N;
  dsgesv_param.nrhs   = NRHS;
  dsgesv_param.d_A    = dA + dA_offset;
  dsgesv_param.lda    = ldda;
  dsgesv_param.d_ipiv = dIPIV;
  dsgesv_param.d_B    = dB + dB_offset;
  dsgesv_param.ldb    = lddb;
  dsgesv_param.d_X    = dX + dX_offset;
  dsgesv_param.ldx    = lddx;
  dsgesv_param.d_work = dworkd;
  dsgesv_param.d_swork= dworks;

  int control_msg     = magma_mic_DSGESV;
  if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
    err = errno;
    printf("scif_send failed with err %d\n", errno);
    fflush(stdout);
  }

  if ((err = scif_send(gEpd, &dsgesv_param, sizeof(dsgesv_param), 1)) <= 0) {
    err = errno;
    printf("scif_send failed with err %d\n", errno);
    fflush(stdout);
  }

  magma_mic_synchronize_kernels();

  return MAGMA_SUCCESS;
}


#endif // HAVE_MIC
