/*
 *   -- micMAGMA (version 1.2.0) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      April 2012
 *
 * @author Azzam Haidar
 * @author Khairul Kabir
 *
 * @generated from magma_z_server.cpp normal z -> c, Sat May 31 21:27:53 2014
 */

#include <mkl.h>

#include <mkl_trans.h>
#include <scif.h>
#include <fcntl.h>
#include <offload.h>

//#ifdef __MIC__
//#define _M512_TYPED
//#include <lmmintrin.h>
//#endif
#include <immintrin.h>


#include "magma.h"
//#include "magma_c_bindings.h"
#include "magma_c_server.h"

#define min(a, b) ((a) < (b) ? (a) : (b))


//=========================================================================================
void call_ccopy(magma_mic_ccopy_param param)
{
    magma_int_t n, incx, incy;
    magmaFloatComplex  *x, *y;
    
    n      = param.n;
    x      = (magmaFloatComplex *)param.x;
    incx   = param.incx;
    y      = (magmaFloatComplex *)param.y;
    incy   = param.incy;
    
    /* sequntial version I think the compiler is optimizing it */
    magma_int_t ix=0, iy=0, ii, chunk=1, nthreads, tid;
    if (incx < 0) ix = (-n+1)*incx;
    if (incy < 0) iy = (-n+1)*incy;
    for (magma_int_t i=0; i < n; i++) {
           y[iy] = x[ix];
           ix += incx;
           iy += incy;
    }

    /* parallel version, where I use openmp to parallelize the copy */
 /* not tested if incx or incy < 0
    #pragma omp parallel
    {
        magma_int_t ix=0, iy=0, ii, nthreads, tid;
        tid      = omp_get_thread_num();
        nthreads = omp_get_num_threads();
        if (incx < 0) ix = (-n+1)*incx;
        if (incy < 0) iy = (-n+1)*incy;
        ix = ix + tid*incx;
        iy = iy + tid*incy;
        for (ii=0; ii < n; ii += nthreads) {
            if ((ii+tid) < n) {
                y[iy] = x[ix];
                ix = ix + incx * nthreads;
                iy = iy + incy * nthreads;
            }
        }
    }
*/
    // mkl_ccopy is not optimized yet and thus giving bad perf.
    // it is better to use the handcode as the compiler is intelligent and
    // optimizes the code better. Azzam
    //ccopy( &n, (MKL_Complex8 *)x, &incx, (MKL_Complex8 *)y, &incy);
}


//=========================================================================================
//magmablas our chemm only left implemented here Azzam
//=========================================================================================
void magmablas_chemm_mic(
    magma_side_t side, magma_uplo_t uplo, magma_int_t n, magma_int_t k, magma_int_t nb,
    magmaFloatComplex alpha, magmaFloatComplex *a, magma_int_t lda,
                              magmaFloatComplex *b, magma_int_t ldb,
    magmaFloatComplex beta,  magmaFloatComplex *c, magma_int_t ldc, magma_int_t agboff)
{
    #define dA(i, j) (a + (i) + (j)*lda)
    #define dB(i, j) (b + (i) + (j)*ldb)
    #define dC(i, j) (c + (i) + (j)*ldc)

    const magmaFloatComplex c_one = MAGMA_C_ONE;
    magmaFloatComplex newbeta = beta;
    magma_int_t m, ib, ioff, blockoffset;
    // loop over all blocks and does A*A'
    // blockoffset is offset within first block; for subsequent blocks it is 0
    blockoffset = agboff % nb;
    for( magma_int_t i = 0; i < n; i += ib ) {
        ib     = min( nb-blockoffset, n-i );  // block size
        /*
        // Symmetrize the upper part of this block
        for (magma_int_t col=0; col < ib; col++) {
            for (magma_int_t row=0; row < col; row++) {
                *dA(i+row, i+col) =  MAGMA_C_CNJG( *(dA(i+col, i+row)) );
            }
        }
        */
        // C[i:n,0] += alpha * A[i:n,i] * B[i,0]
        /*
        m = n - i -ib;
        chemm( lapack_const(side), lapack_const(uplo),
              &ib, &k, (const MKL_Complex8 *)&alpha, (MKL_Complex8 *)(dA(i, i)), &lda,
              (MKL_Complex8 *)(dB(i,0)), &ldb, (const MKL_Complex8 *)&newbeta,
              (MKL_Complex8 *)(dC(i,0)), &ldc);

        cgemm( lapack_const(MagmaNoTrans), lapack_const(MagmaNoTrans), &m, &k, &ib,
               (const MKL_Complex8 *)&alpha,   (MKL_Complex8 *)(dA(ioff+ib, ioff)), &lda,
                                                (MKL_Complex8 *)(dB(i,0)),           &ldb,
               (const MKL_Complex8 *)&newbeta, (MKL_Complex8 *)(dC(i+ib,0)),        &ldc );
        */
        m = n - i;
        cgemm( lapack_const(MagmaNoTrans), lapack_const(MagmaNoTrans), &m, &k, &ib,
               (const MKL_Complex8 *)&alpha,   (MKL_Complex8 *)(dA(i,i)), &lda,
                                                (MKL_Complex8 *)(dB(i,0)), &ldb,
               (const MKL_Complex8 *)&newbeta, (MKL_Complex8 *)(dC(i,0)), &ldc );
        

        cgemm( lapack_const(MagmaConjTrans), lapack_const(MagmaNoTrans), &i, &k, &ib,
               (const MKL_Complex8 *)&alpha,   (MKL_Complex8 *)(dA(i,0)), &lda,
                                                (MKL_Complex8 *)(dB(i,0)), &ldb,
               (const MKL_Complex8 *)&newbeta, (MKL_Complex8 *)(dC(0,0)), &ldc );

        newbeta = c_one;
        blockoffset = 0;
    }
    
    #undef dA
    #undef dB
    #undef dC
}

//=========================================================================================
void call_chemm(magma_mic_chemm_param param)
{
    magma_side_t side;
    magma_uplo_t uplo;
    magma_int_t m, n, lda, ldb, ldc, agboff;
    magmaFloatComplex alpha, beta, *a, *b, *c;

    side   = param.side;
    uplo   = param.uplo;
    m      = param.m;
    n      = param.n;
    alpha  = param.alpha;
    a      = (magmaFloatComplex *)param.a;
    lda    = param.lda;
    b      = (magmaFloatComplex *)param.b;
    ldb    = param.ldb;
    beta   = param.beta;
    c      = (magmaFloatComplex *)param.c;
    ldc    = param.ldc;
    agboff = param.agboff;
    
    if ((side == MagmaLeft) && (uplo == MagmaLower)) {
        //printf("calling AZZAM SYMM m %d   n %d   agboff %d\n", m, n, agboff);
        //Azzam: calling only the Left Lower. need to be developed for all cases
        magma_int_t nb = 960;
        magmablas_chemm_mic(side, uplo,
                         m, n, nb, alpha, a, lda, b, ldb,
                         beta, c, ldc, agboff);
    } else {
        //printf("calling MKL SYMM m %d   n %d \n", m, n);
        chemm( lapack_const(side), lapack_const(uplo),
               &m, &n, (const MKL_Complex8 *)&alpha, (MKL_Complex8 *)a, &lda,
               (MKL_Complex8 *)b, &ldb, (const MKL_Complex8 *)&beta,
               (MKL_Complex8 *)c, &ldc);
    }
}

//=========================================================================================
void call_cgemm(magma_mic_cgemm_param param)
{
    magma_trans_t transa, transb;
    magma_int_t m, n, k, lda, ldb, ldc;
    magmaFloatComplex alpha, beta, *a, *b, *c;
    
    transa = param.transa;
    transb = param.transb;
    m      = param.m;
    n      = param.n;
    k      = param.k;
    alpha  = param.alpha;
    a      = (magmaFloatComplex *)param.a;
    lda    = param.lda;
    b      = (magmaFloatComplex *)param.b;
    ldb    = param.ldb;
    beta   = param.beta;
    c      = (magmaFloatComplex *)param.c;
    ldc    = param.ldc;

    //printf("CGEMM: %c %c %d %d %d %f 0x%lx %d 0x%lx %d %f 0x%lx %d\n", transa, transb, m, n, k, alpha, (off_t)a, lda, (off_t)b, ldb, beta, (off_t)c, ldc);

    cgemm( lapack_const(transa), lapack_const(transb),
           &m, &n, &k, (const MKL_Complex8 *)&alpha, (MKL_Complex8 *)a, &lda,
           (MKL_Complex8 *)b, &ldb, (const MKL_Complex8 *)&beta,
           (MKL_Complex8 *)c, &ldc);
}

//=========================================================================================
void call_cgemv(magma_mic_cgemv_param param)
{
    magma_trans_t transa;
    magma_int_t m, n, lda, incx, incy;
    magmaFloatComplex alpha, beta, *a, *x, *y;
    
    transa = param.transa;
    m      = param.m;
    n      = param.n;
    alpha  = param.alpha;
    a      = (magmaFloatComplex *)param.a;
    lda    = param.lda;
    x      = (magmaFloatComplex *)param.x;
    incx   = param.incx;
    beta   = param.beta;
    y      = (magmaFloatComplex *)param.y;
    incy   = param.incy;
    
    //printf("CGEMV: %c %c %d %d %d %f 0x%lx %d 0x%lx %d %f 0x%lx %d\n", transa, transb, m, n, k, alpha, (off_t)a, lda, (off_t)b, ldb, beta, (off_t)c, ldc);
    
    cgemv( lapack_const(transa), &m, &n, (const MKL_Complex8 *)&alpha,
           (MKL_Complex8 *)a, &lda, (MKL_Complex8 *)x, &incx,
           (const MKL_Complex8 *)&beta, (MKL_Complex8 *)y, &incy);
}

//=========================================================================================
void call_chemv(magma_mic_chemv_param param)
{
    magma_uplo_t uplo;
    magma_int_t n, lda, incx, incy;
    magmaFloatComplex alpha, beta, *a, *x, *y;
    
    uplo   = param.uplo;
    n      = param.n;
    alpha  = param.alpha;
    a      = (magmaFloatComplex *)param.a;
    lda    = param.lda;
    x      = (magmaFloatComplex *)param.x;
    incx   = param.incx;
    beta   = param.beta;
    y      = (magmaFloatComplex *)param.y;
    incy   = param.incy;
    
    chemv( lapack_const(uplo), &n, (const MKL_Complex8 *)&alpha,
           (MKL_Complex8 *)a, &lda, (MKL_Complex8 *)x, &incx,
           (const MKL_Complex8 *)&beta, (MKL_Complex8 *)y, &incy);
}

//=========================================================================================
void call_csymv2(magma_mic_csymv2_param param)
{
    magma_uplo_t uplo;
    magma_int_t n, lda, incx, incy;
    magmaFloatComplex alpha, beta, *a, *x, *y;
    
    uplo   = param.uplo;
    n      = param.n;
    alpha  = param.alpha;
    a      = (magmaFloatComplex *)param.a;
    lda    = param.lda;
    x      = (magmaFloatComplex *)param.x;
    incx   = param.incx;
    beta   = param.beta;
    y      = (magmaFloatComplex *)param.y;
    incy   = param.incy;
    
    csymv( lapack_const(uplo), &n, (const MKL_Complex8 *)&alpha,
           (MKL_Complex8 *)a, &lda, (MKL_Complex8 *)x, &incx,
           (const MKL_Complex8 *)&beta, (MKL_Complex8 *)y, &incy);
}

//=========================================================================================
void call_ctrsm(magma_mic_ctrsm_param param)
{
    magma_side_t side;
    magma_uplo_t uplo;
    magma_trans_t transa;
    magma_diag_t diag;
    magma_int_t m, n, lda, ldb;
    magmaFloatComplex alpha, *a, *b;
    
    side   = param.side;
    uplo   = param.uplo;
    transa = param.transa;
    diag   = param.diag;
    m      = param.m;
    n      = param.n;
    alpha  = param.alpha;
    a      = (magmaFloatComplex *)param.a;
    lda    = param.lda;
    b      = (magmaFloatComplex *)param.b;
    ldb    = param.ldb;
    
    //printf("CTRSM: %c %c %c %c %d %d %f 0x%lx %d 0x%lx %d\n", side, uplo, transa, diag, m, n, alpha, (off_t)a, lda, (off_t)b, ldb);
    ctrsm( lapack_const(side), lapack_const(uplo), lapack_const(transa), lapack_const(diag),
           &m, &n, (const MKL_Complex8 *)&alpha, (MKL_Complex8 *)a, &lda,
           (MKL_Complex8 *)b, &ldb);
}

//=========================================================================================
void call_ctrsv(magma_mic_ctrsv_param param)
{
    magma_uplo_t uplo;
    magma_trans_t trans;
    magma_diag_t diag;
    magma_int_t n, lda, incx;
    magmaFloatComplex alpha, *a, *dx;
    
    uplo  = param.uplo;
    trans = param.trans;
    diag  = param.diag;
    n     = param.n;
    a     = (magmaFloatComplex *)param.a;
    lda   = param.lda;
    dx    = (magmaFloatComplex *)param.dx;
    incx  = param.incx;
    
    ctrsv( lapack_const(uplo), lapack_const(trans), lapack_const(diag),
           &n, (MKL_Complex8 *)a, &lda, (MKL_Complex8 *)dx, &incx);
}

//=========================================================================================
void call_ctrmv(magma_mic_ctrmv_param param)
{
    magma_uplo_t uplo;
    magma_trans_t trans;
    magma_diag_t diag;
    magma_int_t n, lda, incx;
    magmaFloatComplex alpha, *a, *dx;
    
    uplo  = param.uplo;
    trans = param.trans;
    diag  = param.diag;
    n     = param.n;
    a     = (magmaFloatComplex *)param.a;
    lda   = param.lda;
    dx    = (magmaFloatComplex *)param.dx;
    incx  = param.incx;
    
    ctrmv( lapack_const(uplo), lapack_const(trans), lapack_const(diag),
           &n, (MKL_Complex8 *)a, &lda, (MKL_Complex8 *)dx, &incx);
}

//=========================================================================================
void call_ctrmm(magma_mic_ctrmm_param param)
{
    magma_side_t  side;
    magma_uplo_t  uplo;
    magma_trans_t transa;
    magma_diag_t  diag;
    magma_int_t m, n, lda, ldb;
    magmaFloatComplex alpha, *a, *b;

    side   = param.side;
    uplo   = param.uplo;
    transa = param.transa;
    diag   = param.diag;
    m      = param.m;
    n      = param.n;
    alpha  = param.alpha;
    a      = (magmaFloatComplex *)param.a;
    lda    = param.lda;
    b      = (magmaFloatComplex *)param.b;
    ldb    = param.ldb;

    //printf("CTRMM: %c %c %c %c %d %d %f 0x%lx %d 0x%lx %d\n", side, uplo, transa, diag, m, n, alpha, (off_t)a, lda, (off_t)b, ldb);
    ctrmm( lapack_const(side), lapack_const(uplo), lapack_const(transa), lapack_const(diag),
           &m, &n, (const MKL_Complex8 *)&alpha, (MKL_Complex8 *)a, &lda,
           (MKL_Complex8 *)b, &ldb);
}

//=========================================================================================
//magmablas our cherk Azzam
//=========================================================================================
void magmablas_cherk_mic0(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k, magma_int_t nb,
    float alpha, magmaFloatComplex *a, magma_int_t lda, magma_int_t aoff,
    float beta,  magmaFloatComplex *c, magma_int_t ldc, magma_int_t offset)
{
    #define dA(i, j) (a + (i) + (j)*lda + (aoff) )
    #define dC(i, j) (c + (i) + (j)*ldc)

    magmaFloatComplex cbeta  = MAGMA_C_MAKE( beta,  0. );
    magmaFloatComplex calpha = MAGMA_C_MAKE( alpha, 0. );

    magma_int_t m, ib, ioff;
    magma_int_t blockoffset = offset % nb;

    // loop over all blocks and does A*A'
    // blockoffset is offset within first block; for subsequent blocks it is 0
    for( magma_int_t i = 0; i < n; i += ib ) {
        ib     = min( nb-blockoffset, n-i );  // block size
        ioff   = i + offset;                  // global index in parent matrix
        // C[i:n,i] += A[i:n,0] * A[i,0]'
        m = n - i;
        cgemm( lapack_const(MagmaNoTrans), lapack_const(MagmaConjTrans), &m, &ib, &k,
               (const MKL_Complex8 *)&calpha, (MKL_Complex8 *)(dA(i,0)),       &lda,
                                               (MKL_Complex8 *)(dA(i,0)),       &lda,
               (const MKL_Complex8 *)&cbeta,  (MKL_Complex8 *)(dC(ioff,ioff)), &ldc );
        blockoffset = 0;
    }
    
    #undef dA
    #undef dC
}

//=========================================================================================
//magmablas our cherk version2
//=========================================================================================
void magmablas_cherk_mic(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k, magma_int_t nb,
    float alpha, magmaFloatComplex *a, magma_int_t lda, magma_int_t aoff,
    float beta,  magmaFloatComplex *c, magma_int_t ldc, magma_int_t offset)
{
    #define dA(i, j) (a + (i) + (j)*lda + (aoff) )
    #define dC(i, j) (c + (i) + (j)*ldc)

    magmaFloatComplex cbeta  = MAGMA_C_MAKE( beta,  0. );
    magmaFloatComplex calpha = MAGMA_C_MAKE( alpha, 0. );

    //if (n <= 3840)
    //        nb = 1200;
    //else
    //  nb = 960;

    magma_int_t m, ib, ioff;
    magma_int_t blockoffset = offset % nb;

    if (n <= 8640) {
        // loop over all blocks and does A*A'
        // blockoffset is offset within first block; for subsequent blocks it is 0
        for( magma_int_t i = 0; i < n; i += ib ) {
            ib     = min( nb-blockoffset, n-i );  // block size
            ioff   = i + offset;                  // global index in parent matrix
            // C[i:n,i] += A[i:n,0] * A[i,0]'
            m = n - i;
            cgemm( lapack_const(MagmaNoTrans), lapack_const(MagmaConjTrans), &m, &ib, &k,
                            (const MKL_Complex8 *)&calpha, (MKL_Complex8 *)(dA(i,0)),       &lda,
                                                            (MKL_Complex8 *)(dA(i,0)),       &lda,
                            (const MKL_Complex8 *)&cbeta,  (MKL_Complex8 *)(dC(ioff,ioff)), &ldc );
            blockoffset = 0;
        }
    } else {
        magma_int_t i_960 = n / (2 * 960);
        magma_int_t ngemm = i_960 * 960;
        magma_int_t nrema = n - ngemm;
        //gemm for square block
        cgemm( lapack_const(MagmaNoTrans), lapack_const(MagmaConjTrans), &ngemm, &nrema, &k,
                        (const MKL_Complex8 *)&calpha, (MKL_Complex8 *)(dA(nrema, 0)), &lda,
                                                        (MKL_Complex8 *)(dA(0, 0)),     &lda,
                        (const MKL_Complex8 *)&cbeta,  (MKL_Complex8 *)(dC(nrema, 0)), &ldc );

        //two syrk call
        magmablas_cherk_mic(MagmaLower, MagmaNoTrans,
                        nrema, k, nb, alpha, a, lda, 0,
                        beta, c, ldc, 0);

        magmablas_cherk_mic(MagmaLower, MagmaNoTrans,
                        ngemm, k, nb, alpha, dA(nrema, 0), lda, 0,
                        beta, dC(nrema, nrema), ldc, 0);
    }
    
    #undef dA
    #undef dC
}

//=========================================================================================
void call_cherk(magma_mic_cherk_param param)
{
    magma_uplo_t uplo;
    magma_trans_t trans;
    magma_int_t n, k, lda, ldc;
    float alpha, beta;
    magmaFloatComplex *a, *c;

    uplo  = param.uplo;
    trans = param.trans;
    n     = param.n;
    k     = param.k;
    alpha = param.alpha;
    a     = (magmaFloatComplex *)param.a;
    lda   = param.lda;
    beta  = param.beta;
    c     = (magmaFloatComplex *)param.c;
    ldc   = param.ldc;

    //printf("CHERK: %c %c %d %d %f 0x%lx %d %f 0x%lx %d\n", uplo, trans, n, k, alpha, (off_t)a, lda, beta, (off_t)c, ldc);

    // call MKL cherk
    //cherk( lapack_const(uplo), lapack_const(trans),
    //       &n, &k, &alpha, (MKL_Complex8 *)a, &lda,
    //       &beta, (MKL_Complex8 *)c, &ldc);
   
     
    magma_int_t nb = 960;
    magmablas_cherk_mic(MagmaLower, MagmaNoTrans,
                     n, k, nb, alpha, a, lda, 0,
                     beta, c, ldc, 0);
}

//=========================================================================================
void magmablas_cherk_mmic(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k,
    float alpha, magmaFloatComplex *a, magma_int_t lda, magma_int_t aoff,
    float beta,  magmaFloatComplex *c, magma_int_t ldc, magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_int_t my_id)
{
    #define dA(i, j) (a + (i) + (j)*lda + (aoff) )
    #define dC(i, j) (c + (i) + (j)*ldc)

    magmaFloatComplex cbeta  = MAGMA_C_MAKE( beta,  0. );
    magmaFloatComplex calpha = MAGMA_C_MAKE( alpha, 0. );

    magma_int_t rows, ib, ioff, iblock, idev, di, s;

    // loop over all blocks
    // Faster to have two loops: first does A*B', second does B*A'
    // blockoffset is offset within first block; for subsequent blocks it is 0
    magma_int_t blockoffset = offset % nb;
    float tt = 0;
    for( magma_int_t i = 0; i < n; i += ib ) {
        ib     = min( nb-blockoffset, n-i );  // block size
        ioff   = i + offset;                  // global index in parent matrix
        iblock = (ioff / nb) / nmic;          // local block id
        idev   = (ioff / nb) % nmic;          // device with this block
        di     = iblock*nb + blockoffset;     // local index in parent matrix

        // C[i:n,i] += A[i:n,0] * B[i,0]'
        //printf( "cgemm  n=%4d, ib=%4d, k=%4d, i=%4d\n", n-i, ib, k, i );
        if (idev == my_id) {
            rows = n - i;
            cgemm( lapack_const(MagmaNoTrans), lapack_const(MagmaConjTrans), &rows, &ib, &k,
                   (const MKL_Complex8 *)&calpha, (MKL_Complex8 *)dA(i,0),     &lda,
                                                   (MKL_Complex8 *)dA(i,0),     &lda,
                   (const MKL_Complex8 *)&cbeta,  (MKL_Complex8 *)dC(ioff,di), &ldc);
        }

        blockoffset = 0;
    }
    
    #undef dA
    #undef dC
}

//=========================================================================================
void call_cherk_mmic(magma_mic_cherk_mmic_param param)
{
    magma_uplo_t uplo;
    magma_trans_t trans;
    magma_int_t n, k, lda, ldb, ldc, aoff, boff, offset, nb, my_id, nmic;
    float alpha, beta;
    magmaFloatComplex *a, *b, *c;

    uplo   = param.uplo;
    trans  = param.trans;
    n      = param.n;
    k      = param.k;
    alpha  = param.alpha;
    a      = (magmaFloatComplex *)param.a;
    lda    = param.lda;
    aoff   = param.aoff;
    beta   = param.beta;
    c      = (magmaFloatComplex *)param.c;
    ldc    = param.ldc;
    offset = param.offset;
    nmic   = param.nmic;
    nb     = param.nb;
    my_id  = param.my_id;

    //printf("CHERK_MMIC: %c %c %d %d %f 0x%lx %d %f 0x%lx %d\n", uplo, trans, n, k, alpha, (off_t)a, lda, beta, (off_t)c, ldc);

    magmablas_cherk_mmic(
        uplo, trans, n, k,
        alpha, a, lda, aoff,
        beta, c, ldc, offset,
        nmic, nb, my_id);
}

//=========================================================================================
//magmablas our cher2k Azzam
//=========================================================================================
void magmablas_cher2k_mic(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k, magma_int_t nb,
    magmaFloatComplex alpha,
    magmaFloatComplex *a, magma_int_t lda,
    magmaFloatComplex *b, magma_int_t ldb,
    float beta,
    magmaFloatComplex *c, magma_int_t ldc)
{
    #define dA(i, j) (a + (i) + (j)*lda)
    #define dB(i, j) (b + (i) + (j)*ldb)
    #define dC(i, j) (c + (i) + (j)*ldc)

    const magmaFloatComplex c_one = MAGMA_C_ONE;
    magmaFloatComplex cbeta    = MAGMA_C_MAKE( beta, 0. );
    magmaFloatComplex cnjalpha = MAGMA_C_CNJG( alpha );

    magma_int_t m, ib;
    // loop over all blocks and does A*A'
    for( magma_int_t i = 0; i < n; i += ib ) {
        ib     = min( nb, n-i );  // block size
        m = n - i;
        //printf("voici m %d     n %d    k %d \n",m,ib,k);
        // C[i:n,i] += A[i:n,0] * A[i,0]'
        cgemm( lapack_const(MagmaNoTrans), lapack_const(MagmaConjTrans), &m, &ib, &k,
               (const MKL_Complex8 *)&alpha, (MKL_Complex8 *)(dA(i,0)), &lda,
                                              (MKL_Complex8 *)(dB(i,0)), &ldb,
               (const MKL_Complex8 *)&cbeta, (MKL_Complex8 *)(dC(i,i)), &ldc );

        // C[i:n,i] += A[i:n,0] * A[i,0]'
        cgemm( lapack_const(MagmaNoTrans), lapack_const(MagmaConjTrans), &m, &ib, &k,
               (const MKL_Complex8 *)&cnjalpha, (MKL_Complex8 *)(dB(i,0)), &ldb,
                                                 (MKL_Complex8 *)(dA(i,0)), &lda,
               (const MKL_Complex8 *)&c_one,    (MKL_Complex8 *)(dC(i,i)), &ldc );
    }
    
    #undef dA
    #undef dB
    #undef dC
}

//=========================================================================================
void call_cher2k(magma_mic_cher2k_param param)
{
    magma_uplo_t uplo;
    magma_trans_t trans;
    magma_int_t n, k, lda, ldb, ldc;
    magmaFloatComplex alpha;
    float beta;
    magmaFloatComplex *a, *b, *c;

    uplo  = param.uplo;
    trans = param.trans;
    n     = param.n;
    k     = param.k;
    alpha = param.alpha;
    a     = (magmaFloatComplex *)param.a;
    lda   = param.lda;
    b     = (magmaFloatComplex *)param.b;
    ldb   = param.ldb;
    beta  = param.beta;
    c     = (magmaFloatComplex *)param.c;
    ldc   = param.ldc;

    //printf("CHER2K: %c %c %d %d %f 0x%lx %d %f 0x%lx %d\n", uplo,trans, n,k, alpha, (off_t)a,lda, beta, (off_t)c, ldc);
   
    if (uplo == MagmaUpper) {
           cher2k( lapack_const(uplo), lapack_const(trans),
                   &n, &k, (MKL_Complex8 *)&alpha,
                           (MKL_Complex8 *)a, &lda,
                           (MKL_Complex8 *)b, &ldb,
                   &beta,  (MKL_Complex8 *)c, &ldc);
    } else {
           magma_int_t nb = 960;
           magmablas_cher2k_mic( MagmaLower, MagmaNoTrans,
                           n, k, nb, alpha, a, lda,
                           b, ldb,
                           beta, c, ldc);
    }
}

//=========================================================================================
//magmablas our cher2k_mmic Azzam
//=========================================================================================
void magmablas_cher2k_mmic(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k,
    magmaFloatComplex alpha,
    magmaFloatComplex *a, magma_int_t lda, magma_int_t aoff,
    magmaFloatComplex *b, magma_int_t ldb, magma_int_t boff,
    float beta,
    magmaFloatComplex *c, magma_int_t ldc,  magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_int_t my_id)
{
    #define dA(i, j) (a + (i) + (j)*lda + (aoff) )
    #define dB(i, j) (b + (i) + (j)*ldb + (boff) )
    #define dC(i, j) (c + (i) + (j)*ldc)

    const magmaFloatComplex c_one = MAGMA_C_ONE;
    magmaFloatComplex cbeta = MAGMA_C_MAKE( beta, 0. );


    magma_int_t rows, ib, ioff, iblock, idev, di, s;

    // loop over all blocks
    // Faster to have two loops: first does A*B', second does B*A'
    // blockoffset is offset within first block; for subsequent blocks it is 0
    magma_int_t blockoffset = offset % nb;
    for( magma_int_t i = 0; i < n; i += ib ) {
        ib     = min( nb-blockoffset, n-i );  // block size
        ioff   = i + offset;                  // global index in parent matrix
        iblock = (ioff / nb) / nmic;          // local block id
        idev   = (ioff / nb) % nmic;          // device with this block
        di     = iblock*nb + blockoffset;     // local index in parent matrix

        // C[i:n,i] += A[i:n,0] * B[i,0]'
        //printf( "cgemm  n=%4d, ib=%4d, k=%4d, i=%4d\n", n-i, ib, k, i );
        if (idev == my_id) {
            rows = n - i;
            cgemm( lapack_const(MagmaNoTrans), lapack_const(MagmaConjTrans), &rows, &ib, &k,
                   (const MKL_Complex8 *)&alpha, (MKL_Complex8 *)dA(i,0),     &lda,
                                                  (MKL_Complex8 *)dB(i,0),     &ldb,
                   (const MKL_Complex8 *)&cbeta, (MKL_Complex8 *)dC(ioff,di), &ldc);
        }

        blockoffset = 0;
    }

    blockoffset = offset % nb;
    alpha = MAGMA_C_CNJG( alpha );
    for( magma_int_t i = 0; i < n; i += ib ) {
        ib     = min( nb-blockoffset, n-i );  // block size
        ioff   = i + offset;                  // global index in parent matrix
        iblock = (ioff / nb) / nmic;          // local block id
        idev   = (ioff / nb) % nmic;          // device with this block
        di     = iblock*nb + blockoffset;     // local index in parent matrix

        // C[i:n,i] += B[i:n,0] * A[i,0]'
        //printf( "cgemm  n=%4d, ib=%4d, k=%4d, i=%4d\n", n-i, ib, k, i );
        if (idev == my_id) {
            rows = n - i;
            cgemm( lapack_const(MagmaNoTrans), lapack_const(MagmaConjTrans), &rows, &ib, &k,
                   (const MKL_Complex8 *)&alpha, (MKL_Complex8 *)dB(i,0),     &ldb,
                                                  (MKL_Complex8 *)dA(i,0),     &lda,
                   (const MKL_Complex8 *)&c_one, (MKL_Complex8 *)dC(ioff,di), &ldc);
        }
        blockoffset = 0;
    }
    
    #undef dA
    #undef dB
    #undef dC
}

//=========================================================================================
void call_cher2k_mmic(magma_mic_cher2k_mmic_param param)
{
    magma_uplo_t uplo;
    magma_trans_t trans;
    magma_int_t n, k, lda, ldb, ldc, aoff, boff, offset, nb, my_id, nmic;
    magmaFloatComplex alpha;
    float beta;
    magmaFloatComplex *a, *b, *c;

    uplo   = param.uplo;
    trans  = param.trans;
    n      = param.n;
    k      = param.k;
    alpha  = param.alpha;
    a      = (magmaFloatComplex *)param.a;
    lda    = param.lda;
    aoff   = param.aoff;
    b      = (magmaFloatComplex *)param.b;
    ldb    = param.ldb;
    boff   = param.boff;
    beta   = param.beta;
    c      = (magmaFloatComplex *)param.c;
    ldc    = param.ldc;
    offset = param.offset;
    nmic   = param.nmic;
    nb     = param.nb;
    my_id  = param.my_id;

    //printf("CHER2K_MMIC: %c %c %d %d %f 0x%lx %d %f 0x%lx %d\n", uplo, trans, n, k, alpha, (off_t)a, lda, beta, (off_t)c, ldc);

    magmablas_cher2k_mmic(
        uplo, trans, n, k,
        alpha, a, lda, aoff,
        b, ldb, boff,
        beta, c, ldc, offset,
        nmic, nb, my_id);
}

//=========================================================================================
void call_clarft(magma_mic_clarft_param param)
{
    magma_direct_t direct;
    magma_storev_t storev;
    magma_int_t n, k, ldv, ldt;
    magmaFloatComplex *v, *tau, *t;
    
    direct = param.direct;
    storev = param.storev;
    n      = param.n;
    k      = param.k;
    v      = (magmaFloatComplex *)param.v;
    ldv    = param.ldv;
    tau    = (magmaFloatComplex *)param.tau;
    t      = (magmaFloatComplex *)param.t;
    ldt    = param.ldt;
    
    //printf("CLARFT: %c %c %d %d 0x%lx %d ox%lx 0x%lx %d\n", direct, storev, n, k, (off_t)v, ldv, (off_t)tau, (off_t)t, ldt);
    
    clarft( lapack_const(direct), lapack_const(storev),
            &n, &k, (MKL_Complex8 *)v, &ldv,
            (MKL_Complex8 *)tau, (MKL_Complex8 *)t, &ldt);
}

//=========================================================================================
void call_clarfb(magma_mic_clarfb_param param)
{
    magma_side_t   side;
    magma_trans_t  trans;
    magma_direct_t direct;
    magma_storev_t storev;
    magma_int_t m, n, k, ldv, ldt, ldc, ldwork;
    magmaFloatComplex *v, *t, *c, *work;

    side   = param.side;
    trans  = param.trans;
    direct = param.direct;
    storev = param.storev;
    m      = param.m;
    n      = param.n;
    k      = param.k;
    v      = (magmaFloatComplex *)param.v;
    ldv    = param.ldv;
    t      = (magmaFloatComplex *)param.t;
    ldt    = param.ldt;
    c      = (magmaFloatComplex *)param.c;
    ldc    = param.ldc;
    work   = (magmaFloatComplex *)param.work;
    ldwork = param.ldwork;

    clarfb( lapack_const(side), lapack_const(trans), lapack_const(direct), lapack_const(storev),
            &m, &n, &k, (MKL_Complex8 *)v, &ldv, (MKL_Complex8 *)t, &ldt,
            (MKL_Complex8 *)c, &ldc, (MKL_Complex8 *)work, &ldwork);
}

//=========================================================================================
//magmablas our CLARFB based on GEMMs Azzam
//=========================================================================================
void call_clarfb_gemm(magma_mic_clarfb_gemm_param param)
{
    magma_side_t   side;
    magma_trans_t  trans;
    magma_direct_t direct;
    magma_storev_t storev;
    magma_int_t m, n, k, ldv, ldt, ldc, ldwork, ldworkvt;
    magmaFloatComplex *v, *t, *c, *work, *workvt;
    
    side     = param.side;
    trans    = param.trans;
    direct   = param.direct;
    storev   = param.storev;
    m        = param.m;
    n        = param.n;
    k        = param.k;
    v        = (magmaFloatComplex *)param.v;
    ldv      = param.ldv;
    t        = (magmaFloatComplex *)param.t;
    ldt      = param.ldt;
    c        = (magmaFloatComplex *)param.c;
    ldc      = param.ldc;
    work     = (magmaFloatComplex *)param.work;
    ldwork   = param.ldwork;
    workvt   = (magmaFloatComplex *)param.workvt;
    ldworkvt = param.ldworkvt;
    
    magmaFloatComplex c_zero    = MAGMA_C_ZERO;
    magmaFloatComplex c_one     = MAGMA_C_ONE;
    magmaFloatComplex c_neg_one = MAGMA_C_NEG_ONE;

    /* Function Body */
    if (m <= 0 || n <= 0) {
        return;
    }

    // opposite of trans
    magma_trans_t transt;
    if (trans == MagmaNoTrans)
        transt = MagmaConjTrans;
    else
        transt = MagmaNoTrans;
    
    // whether T is upper or lower triangular
    magma_uplo_t uplo;
    if (direct == MagmaForward)
        uplo = MagmaUpper;
    else
        uplo = MagmaLower;
    
    // whether V is stored transposed or not
    magma_int_t notransV, transV;
    if (storev == MagmaColumnwise) {
        notransV = MagmaNoTrans;
        transV   = MagmaConjTrans;
    }
    else {
        notransV = MagmaConjTrans;
        transV   = MagmaNoTrans;
    }

    if ( side == MagmaLeft ) {
        // Form H C or H' C
        // Comments assume H C.
        // When forming H' C, T gets transposed via transt for m >= n or by trans for m < n.
        
        // W = C' V
        cgemm( lapack_const(MagmaConjTrans), lapack_const(notransV),
               &n, &k, &m,
               (const MKL_Complex8 *)&c_one,  (MKL_Complex8 *)c,    &ldc,
                                               (MKL_Complex8 *)v,    &ldv,
               (const MKL_Complex8 *)&c_zero, (MKL_Complex8 *)work, &ldwork);

        if (m < n) {
            // W2 = V T
            cgemm( lapack_const(notransV), lapack_const(trans),
                   &m, &k, &k,
                   (const MKL_Complex8 *)&c_one,  (MKL_Complex8 *)v,      &ldv,
                                                   (MKL_Complex8 *)t,      &ldt,
                   (const MKL_Complex8 *)&c_zero, (MKL_Complex8 *)workvt, &ldworkvt);
                        
            // C = C - W2 W' = C - V T V' C = (I - V T V') C = H C
            cgemm( lapack_const(MagmaNoTrans), lapack_const(MagmaConjTrans),
                   &m, &n, &k,
                   (const MKL_Complex8 *)&c_neg_one, (MKL_Complex8 *)workvt, &ldworkvt,
                                                      (MKL_Complex8 *)work,   &ldwork,
                   (const MKL_Complex8 *)&c_one,     (MKL_Complex8 *)c,      &ldc);
        } else {
            // W2 = W T' = C' V T'
            cgemm( lapack_const(MagmaNoTrans), lapack_const(transt),
                   &n, &k, &k,
                   (const MKL_Complex8 *)&c_one,  (MKL_Complex8 *)work,   &ldwork,
                                                   (MKL_Complex8 *)t,      &ldt,
                   (const MKL_Complex8 *)&c_zero, (MKL_Complex8 *)workvt, &ldworkvt);
                                                 
            // C = C - V W2' = C - V T V' C = (I - V T V') C = H C
            cgemm( lapack_const(notransV), lapack_const(MagmaConjTrans),
                   &m, &n, &k,
                   (const MKL_Complex8 *)&c_neg_one, (MKL_Complex8 *)v,      &ldv,
                                                      (MKL_Complex8 *)workvt, &ldworkvt,
                   (const MKL_Complex8 *)&c_one,     (MKL_Complex8 *)c,      &ldc);
        }
    }
    else {
        // Form C H or C H'
        // Comments assume C H.
        // When forming C H', T gets transposed via trans.
        
        // W = C V
        cgemm( lapack_const(MagmaNoTrans), lapack_const(notransV),
               &m, &k, &n,
               (const MKL_Complex8 *)&c_one,  (MKL_Complex8 *)c,    &ldc,
                                               (MKL_Complex8 *)v,    &ldv,
               (const MKL_Complex8 *)&c_zero, (MKL_Complex8 *)work, &ldwork);
        if (m <= n) {
            // W2 = W T = C V T
            cgemm( lapack_const(MagmaNoTrans), lapack_const(trans),
                   &m, &k, &k,
                   (const MKL_Complex8 *)&c_one,  (MKL_Complex8 *)work,   &ldwork,
                                                   (MKL_Complex8 *)t,      &ldt,
                   (const MKL_Complex8 *)&c_zero, (MKL_Complex8 *)workvt, &ldworkvt);
            
            // C = C - W2 V' = C - C V T V' = C (I - V T V') = C H
            cgemm( lapack_const(MagmaNoTrans), lapack_const(transV),
                   &m, &n, &k,
                   (const MKL_Complex8 *)&c_neg_one, (MKL_Complex8 *)work, &ldwork,
                                                      (MKL_Complex8 *)v,    &ldv,
                   (const MKL_Complex8 *)&c_one,     (MKL_Complex8 *)c,    &ldc);
        } else {
            // W2 = T V'
            cgemm( lapack_const(trans), lapack_const(transV),
                   &k, &n, &k,
                   (const MKL_Complex8 *)&c_one,  (MKL_Complex8 *)t,      &ldt,
                                                   (MKL_Complex8 *)v,      &ldv,
                   (const MKL_Complex8 *)&c_zero, (MKL_Complex8 *)workvt, &ldworkvt);
            // C = C - W W2 = C - C V T V' = C (I - V T V') = C H
            cgemm( lapack_const(MagmaNoTrans), lapack_const(MagmaNoTrans),
                   &m, &n, &k,
                   (const MKL_Complex8 *)&c_neg_one, (MKL_Complex8 *)work,   &ldwork,
                                                      (MKL_Complex8 *)workvt, &ldworkvt,
                   (const MKL_Complex8 *)&c_one,     (MKL_Complex8 *)c,      &ldc);
        }
    }
}

//=========================================================================================
void call_claset(magma_mic_claset_param param)
{
    magma_uplo_t uplo;
    magma_int_t m, n, lda;
    magmaFloatComplex alpha, beta;
    magmaFloatComplex *dA;

    uplo  = param.uplo;
    m     = param.m;
    n     = param.n;
    alpha = param.alpha;
    beta  = param.beta;
    dA    = (magmaFloatComplex *)param.dA;
    lda   = param.lda;

    claset( lapack_const(uplo), &m, &n,
            (const MKL_Complex8 *)&alpha, (const MKL_Complex8 *)&beta,
            (MKL_Complex8 *)dA, &lda);
}

//=========================================================================================
void call_ctranspose(magma_mic_ctranspose_param param)
{
        /******************************************************
         * _mm512_load_pd -- for loading
         * _mm512_i32lo[ext]scatter_pd
         * _mm512_i32loscatter_pd is the simplified version of ...ext...
         * they do the same thing, the latter needs less arguments.
         * **************************************************/
        //        printf("entered stranpose\n");
        //        fflush(stdout);
        /*
           float *odata, *idata;
           magma_int_t ldo,ldi, m, n;
           magma_int_t num_threads;
           magma_int_t tid;
           magma_int_t index[16], i;
           __M512C _reg;
           __m512i _index;
           float *my_head, *loop_head, *dest_head;
           magma_int_t num_cols, num_cols_g, col, iter, num_iter, num_rows_g, num_rows;

           odata = (float *) param.odata;
           ldo   = param.ldo;
           idata = (float *) param.idata;
           ldi   = param.ldi;
           m     = param.m;
           n     = param.n;

           if (m == 0 || n == 0) {
           return;
           }
           if (m <= n) {
           for (i = 0; i < 16; i++) {
           index[i] = i*CREG_WIDTH*ldo;
           }
        // 62 cores, 4 hardware threads per core. Leave out one core for the OS
        // (62 - 1)*4 = 244

        num_threads = 244;
        num_cols_g =        (n + num_threads - 1)/num_threads;
        num_iter = (int) m*1.0/CREG_WIDTH;
        _index = _mm512_set_epi32(index[15], index[14], index[13], index[12],
        index[11], index[10], index[9], index[8], index[7], index[6],
        index[5], index[4], index[3], index[2], index[1], index[0]);

#pragma omp parallel num_threads(num_threads) private(tid, col, num_cols, iter,\
my_head, loop_head, dest_head, _reg)
{
tid = omp_get_thread_num();
num_cols = num_cols_g;
if (tid == (num_threads - 1) && (n%num_cols_g != 0)) {
num_cols = n%num_cols_g;
}
// starting point of my section, constant for each thread
my_head = idata + ldi*(tid*num_cols_g);
for (iter = 0; iter < num_iter; iter++) { // loop inside each column
for (col = 0; col < num_cols; col++) { // loop over each column
if (tid*num_cols_g + col < n) {
        // starting point of the inner loop
        loop_head = my_head + col*ldi + iter*CREG_WIDTH;
        dest_head = odata + (iter*CREG_WIDTH)*ldo + (tid*num_cols_g + col);
        // load 8 real floating point nubmers from address
        // mt into a float64 vector.
        _reg = _MM512_LOAD_PPC( (void const *)loop_head);
        //                                        _mm512_load_pd( (void const *)loop_head);

        // this intrinsic needs int32 indices, but a 512 register
        // contains 16 int32 integers, and there is only one intrinsic
        // to set all 16 positoins of the register. So _index contains
        // 16 int32 integers.
        _MM512_I32LOEXTSCATTER_PPC((void *) dest_head, _index, _reg,
        _MM_DOWNCONV_PC_NONE, 1, _MM_HINT_NONE);
        }
        }
        }
        }
        } else {
        // init index[], it's for scattering the entries to the dest matrix
        for (i = 0; i < 16; i++) {
        index[i] = i*CREG_WIDTH*ldo;
        }

        // 62 cores, 4 hardware threads per core. Leave out one core for the OS
        // (62 - 1)*4 = 244
        num_threads = 244;
        // number of rows in each group
        num_rows_g = (m/CREG_WIDTH + num_threads - 1)/num_threads*CREG_WIDTH;
        _index = _mm512_set_epi32(index[15], index[14], index[13], index[12],
                        index[11], index[10], index[9], index[8], index[7], index[6],
                        index[5], index[4], index[3], index[2], index[1], index[0]);

#pragma omp parallel num_threads(num_threads) private(tid, col, num_rows, iter,\
                loop_head, dest_head, _reg, num_iter)
        {
                tid = omp_get_thread_num();
                // compute the number of rows I need to work on
                num_rows = num_rows_g;
                if (tid == (num_threads - 1) && (m%num_rows_g != 0)) {
                        num_rows = m%num_rows_g;
                }
                num_iter = (int) num_rows*1.0/CREG_WIDTH;

                for (iter = 0; iter < num_iter; iter++) {
                        if (tid*num_rows_g + iter*CREG_WIDTH < m) {
                                for (col = 0; col < n; col++) {
                                        loop_head = idata + (tid*num_rows_g + iter*CREG_WIDTH) + ldi*col;
                                        dest_head = odata + ldo*(tid*num_rows_g + iter*CREG_WIDTH) + col;

                                        _reg = _MM512_LOAD_PPC((void const *) loop_head);
                                        _MM512_I32LOEXTSCATTER_PPC((void *) dest_head, _index, _reg,
                                                        _MM_DOWNCONV_PC_NONE, 1, _MM_HINT_NONE);
                                }
                        }
                }
        }
}
*/
return;
}

//=========================================================================================
void call_ctranspose_mkl(magma_mic_ctranspose_param param)
{
    magmaFloatComplex *odata, *idata;
    magma_int_t ldo,ldi, m, n;
    magma_int_t num_threads;
    magma_int_t tid;
    magma_int_t index[16], i;
    //  _M512 _reg;
    //  _M512I _index;
    magmaFloatComplex *my_head, *loop_head, *dest_head;
    magma_int_t num_cols, num_cols_g, col, iter, num_iter;
    
    odata = (magmaFloatComplex *) param.odata;
    ldo   = param.ldo;
    idata = (magmaFloatComplex *) param.idata;
    ldi   = param.ldi;
    m     = param.m;
    n     = param.n;
    magmaFloatComplex c_one = MAGMA_C_ONE;
    
    mkl_comatcopy('C', 'T', m, n, *((MKL_Complex8 *)(&c_one)), (MKL_Complex8 *)idata, ldi,
                    (MKL_Complex8 *)odata, ldo);
}

//=========================================================================================
void call_claswp(magma_mic_claswp_param param)
{
    magma_int_t n, k1, k2, lda, incx;
    magmaFloatComplex *d_A;
    magma_int_t *d_ipiv;
                     
    n      = param.n;
    d_A    = (magmaFloatComplex *) param.d_A;
    lda    = param.lda;
    k1     = param.k1;
    k2     = param.k2;
    d_ipiv = (magma_int_t *) param.d_ipiv;
    incx   = param.incx;
    
    //printf("n: %d lda: %d k1: %d k2: %d incx: %d\n", n, lda, k1, k2, incx);
    claswp(&n, (MKL_Complex8 *)d_A, &lda, &k1, &k2, (magma_int_t *)d_ipiv, &incx);
}

//=========================================================================================
void call_cswap(magma_mic_cswap_param param)
{
    magma_int_t n, lda1, lda2;
    magmaFloatComplex tmp, *dA1T, *dA2T;
                     
    n       = param.n;
    dA1T    = (magmaFloatComplex *) param.dA1T;
    lda1    = param.lda1;
    dA2T    = (magmaFloatComplex *) param.dA2T;
    lda2    = param.lda2;
    
    cswap(&n, (MKL_Complex8 *)dA1T, &lda1, (MKL_Complex8 *)dA2T, &lda2);
}

//=========================================================================================
void call_ctrsmgemm(magma_mic_ctrsmgemm_param param)
{
    float gtrsmflops, ggemmflops, mic_trsmtime, mic_gemmtime;

    magma_side_t side;
    magma_uplo_t uplo;
    magma_trans_t ttransa;
    magma_diag_t diag;
    magma_int_t tm, tn, tlda, tldb;
    magmaFloatComplex talpha, *ta, *tb;

    side    = param.trsm_param.side;
    uplo    = param.trsm_param.uplo;
    ttransa = param.trsm_param.transa;
    diag    = param.trsm_param.diag;
    tm      = param.trsm_param.m;
    tn      = param.trsm_param.n;
    talpha  = param.trsm_param.alpha;
    ta      = (magmaFloatComplex *)param.trsm_param.a;
    tlda    = param.trsm_param.lda;
    tb      = (magmaFloatComplex *)param.trsm_param.b;
    tldb    = param.trsm_param.ldb;

    mic_trsmtime = dsecnd();

    ctrsm( lapack_const(side), lapack_const(uplo), lapack_const(ttransa), lapack_const(diag),
           &tm, &tn, (const MKL_Complex8 *)&talpha, (MKL_Complex8 *)ta, &tlda,
           (MKL_Complex8 *)tb, &tldb);

    mic_trsmtime = dsecnd() - mic_trsmtime;

    gtrsmflops = 0;
    if (side == MagmaLeft)
        gtrsmflops = MICFLOPS_TRSM_L((float)tm, (float)tn) * 1e-9;
    else if (side == MagmaRight)
        gtrsmflops = MICFLOPS_TRSM_R((float)tm, (float)tn) * 1e-9;

    magma_trans_t transa, transb;
    magma_int_t m, n, k, lda, ldb, ldc;
    magmaFloatComplex alpha, beta, *a, *b, *c;

    transa = param.gemm_param.transa;
    transb = param.gemm_param.transb;
    m      = param.gemm_param.m;
    n      = param.gemm_param.n;
    k      = param.gemm_param.k;
    alpha  = param.gemm_param.alpha;
    a      = (magmaFloatComplex *)param.gemm_param.a;
    lda    = param.gemm_param.lda;
    b      = (magmaFloatComplex *)param.gemm_param.b;
    ldb    = param.gemm_param.ldb;
    beta   = param.gemm_param.beta;
    c      = (magmaFloatComplex *)param.gemm_param.c;
    ldc    = param.gemm_param.ldc;

    //printf("CGEMM: %c %c %d %d %d %f 0x%lx %d 0x%lx %d %f 0x%lx %d\n", transa, transb, m, n, k, alpha, (off_t)a, lda, (off_t)b, ldb, beta, (off_t)c, ldc);

    mic_gemmtime = dsecnd();

    cgemm( lapack_const(transa), lapack_const(transb),
           &m, &n, &k, (const MKL_Complex8 *)&alpha, (MKL_Complex8 *)a, &lda,
           (MKL_Complex8 *)b, &ldb, (const MKL_Complex8 *)&beta,
           (MKL_Complex8 *)c, &ldc);

    mic_gemmtime = dsecnd() - mic_gemmtime;

    ggemmflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;

    //printf("DTRSMGEMM: %8.4lf( %8.4lf )\t %8.4lf( %8.4lf )\n", gtrsmflops / mic_trsmtime, mic_trsmtime * 1000, ggemmflops / mic_gemmtime, mic_gemmtime * 1000);
}

//=========================================================================================
