/*
 *   -- micMAGMA (version 1.2.0) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      April 2012
 *
 *   @author Azzam Haidar
 *   @author Khairul Kabir
 *
 *   @generated from magma_z_kernels.h normal z -> s, Sat May 31 21:27:53 2014
 *
 *   This is included in magma_mic_server.cpp,
 *   inside a switch { } block in the do_work() function.
 */

case magma_mic_SCOPY:
{
    //printf("scopy\n");
    magma_mic_scopy_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SCOPY\n");
        return;
    }

    mic_time = dsecnd();
    call_scopy(param);
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_SGEMM:
{
    //printf("sgemm\n");
    magma_mic_sgemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SGEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_sgemm(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    k = param.k;
    gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("sgemm: %d\t%d\t%d\t%8.2lf\t%8.2lf\t%8.2lf\n", m, n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_SSYMM:
{
    //printf("ssymm\n");
    magma_mic_ssymm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSYMM\n");
        return;
    }

    mic_time = dsecnd();
    call_ssymm(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    magma_side_t side = param.side;

    //gflops = MICFLOPS_SYMM((magma_side_t)side, (float)m, (float)n ) * 1e-9;
    //printf("ssymm: %d\t%d\t%g\t%g\t%g\n", m, n, mic_time,gflops, gflops/mic_time);
    break;
}

case magma_mic_SGEMV:
{
    //printf("sgemv\n");
    magma_mic_sgemv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SGEMV\n");
        return;
    }

    mic_time = dsecnd();
    call_sgemv(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    //gflops = MICFLOPS_GEMV((float)m, (float)n) * 1e-9;
    //printf("sgemv: %d\t%d\t%g\t%g\t%g\n", m, n, mic_time,
    //        gflops, gflops/mic_time);
    break;
}

case magma_mic_SSYMV:
{
    //printf("ssymv\n");
    magma_mic_ssymv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSYMV\n");
        return;
    }

    mic_time = dsecnd();
    call_ssymv(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_SSYMV2:
{
    //printf("ssymv2\n");
    magma_mic_ssymv2_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSYMV\n");
        return;
    }

    mic_time = dsecnd();
    call_ssymv2(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_STRSM:
{
    //printf("strsm\n");
    magma_mic_strsm_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for STRSM\n");
        return;
    }

    mic_time = dsecnd();
    call_strsm(param);
    mic_time = dsecnd() - mic_time;

    side = param.side;
    m = param.m;
    n = param.n;

    if (side == MagmaLeft)
        gflops = MICFLOPS_TRSM_L((float)m, (float)n) * 1e-9;
    else
        gflops = MICFLOPS_TRSM_R((float)m, (float)n) * 1e-9;

    //printf("strsm: %c\t%d\t%d\t%f\t%f\t%f\n", side, m, n, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_STRSV:
{
    //printf("strsv\n");

    magma_mic_strsv_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for STRSV\n");
        return;
    }
    
    mic_time = dsecnd();
    call_strsv(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}

case magma_mic_STRMV:
{
    //printf("strmv\n");
    
    magma_mic_strmv_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for STRMV\n");
        return;
    }
    
    mic_time = dsecnd();
    call_strmv(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}

case magma_mic_STRMM:
{
    //printf("strmm\n");
    magma_mic_strmm_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for STRMM\n");
        return;
    }

    mic_time = dsecnd();
    call_strmm(param);
    mic_time = dsecnd() - mic_time;

    side = param.side;
    m = param.m;
    n = param.n;

    //if (side == MagmaLeft)
    //    gflops = MICFLOPS_TRSM_L((float)m, (float)n) * 1e-9;
    //else
    //    gflops = MICFLOPS_TRSM_R((float)m, (float)n) * 1e-9;

    //printf("strmm: %c\t%d\t%d\t%f\t%f\t%f\n", side, m, n, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_SSYRK:
{
    //printf("ssyrk\n");
    magma_mic_ssyrk_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSYRK\n");
        return;
    }

    mic_time = dsecnd();
    call_ssyrk(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("ssyrk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_SSYRK_MMIC:
{
    //printf("ssyrk\n");
    magma_mic_ssyrk_mmic_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSYRK_MMIC\n");
        return;
    }

    mic_time = dsecnd();
    call_ssyrk_mmic(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("ssyrk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}


case magma_mic_SSYR2K:
{
    //printf("ssyr2k\n");
    magma_mic_ssyr2k_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSYR2K\n");
        return;
    }

    mic_time = dsecnd();
    call_ssyr2k(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYR2K((float)n, (float)k) * 1e-9;
    //printf("ssyr2k: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_SSYR2K_MMIC:
{
    //printf("ssyr2k_mmic\n");
    magma_mic_ssyr2k_mmic_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSYR2K_MMIC\n");
        return;
    }

    //mic_time = dsecnd();
    call_ssyr2k_mmic(param);
    //mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("ssyrk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_SLARFT:
{
    //printf("slarft\n");
    magma_mic_slarft_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SLARFT\n");
        return;
    }

    //mic_time = dsecnd();
    call_slarft(param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("slarft: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_SLARFB:
{
    //printf("slarfb\n");
    magma_mic_slarfb_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SLARFB\n");
        return;
    }

    //mic_time = dsecnd();
    call_slarfb(param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("slarfb: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_SLARFB_GEMM:
{
    //printf("slarfb\n");
    magma_mic_slarfb_gemm_param slarfb_gemm_param;
    if (mic_recv(&slarfb_gemm_param, sizeof(slarfb_gemm_param)) == MIC_FAILURE) {
        printf("scif_recv failed for SLARFB_GEMM\n");
        return;
    }

    //mic_time = dsecnd();
    call_slarfb_gemm(slarfb_gemm_param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("slarfb: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_STRSM_GEMM:
{
    //printf("strsmgemm\n");
    magma_mic_strsmgemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for STRSM_GEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_strsmgemm(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_SLASET:
{
    magma_mic_slaset_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SLASET\n");
        return;
    }

    call_slaset(param);

    break;
}

case magma_mic_SLASWP:
{
    magma_mic_slaswp_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SLASWP\n");
        return;
    }

    call_slaswp(param);

    break;
}

case magma_mic_SSWAP:
{
    magma_mic_sswap_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSWAP\n");
        return;
    }

    call_sswap(param);

    break;
}

case magma_mic_STRANSPOSE:
{
    magma_mic_stranspose_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for STRANSPOSE\n");
        return;
    }

    call_stranspose(param);
    break;
}
