/*
 *  -- MAGMA (version 1.2.0) --
 *     Univ. of Tennessee, Knoxville
 *     Univ. of California, Berkeley
 *     Univ. of Colorado, Denver
 *     April 2012
 *
 * @generated from testing_zgemm.cpp normal z -> c, Sat May 31 21:27:54 2014
 *
 **/
// includes, system
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

// includes, project
#include "flops.h"
#include "magma.h"
#include "magma_lapack.h"
#include "testings.h"


/* ////////////////////////////////////////////////////////////////////////////
   -- Testing cgemm
*/
int main( int argc, char** argv)
{
    /* Initialize */
    magma_queue_t  queue;
    magma_device_t device;
    magma_int_t num = 0;
    magma_int_t err;

    magma_init();
    err = magma_get_devices( &device, 1, &num );
    if ( err != MAGMA_SUCCESS or num < 1 ) {
      fprintf( stderr, "magma_get_devices failed: %d\n", (int) err );
      exit(-1);
    }
    err = magma_queue_create( device, &queue );
    if ( err != MAGMA_SUCCESS ) {
      fprintf( stderr, "magma_queue_create failed: %d\n", (int) err );
      exit(-1);
    }

    real_Double_t   gflops, mic_perf, mic_time, cpu_perf, cpu_time;
    float          mic_error, Cnorm, work[1];
    magma_int_t M, N, K;
    magma_int_t Am, An, Bm, Bn;
    magma_int_t sizeA, sizeB, sizeC;
    magma_int_t lda, ldb, ldc, ldda, lddb, lddc;
    magma_int_t ione     = 1;
    magma_int_t ISEED[4] = {0,0,0,1};
    
    magmaFloatComplex *h_A, *h_B, *h_C, *h_Cmic;
    magmaFloatComplex *d_A, *d_B, *d_C;
    magmaFloatComplex c_neg_one = MAGMA_C_NEG_ONE;
    magmaFloatComplex alpha = MAGMA_C_MAKE(  0.29, -0.86 );
    magmaFloatComplex beta  = MAGMA_C_MAKE( -0.48,  0.38 );
    
    magma_opts opts;
    parse_opts( argc, argv, &opts );
    
    printf("    M     N     K   MIC Gflop/s (ms)   CPU Gflop/s (ms)  MIC error\n");
    printf("==================================================================\n");
    for( int i = 0; i < opts.ntest; ++i ) {
        for( int iter = 0; iter < opts.niter; ++iter ) {
            M = opts.msize[i];
            N = opts.nsize[i];
            K = opts.ksize[i];
            gflops = FLOPS_CGEMM( M, N, K ) / 1e9;

            if ( opts.transA == MagmaNoTrans ) {
                lda = Am = M;
                An = K;
            } else {
                lda = Am = K;
                An = M;
            }
            
            if ( opts.transB == MagmaNoTrans ) {
                ldb = Bm = K;
                Bn = N;
            } else {
                ldb = Bm = N;
                Bn = K;
            }
            ldc = M;
            
            ldda = ((lda+31)/32)*32;
            lddb = ((ldb+31)/32)*32;
            lddc = ((ldc+31)/32)*32;
            
            sizeA = lda*An;
            sizeB = ldb*Bn;
            sizeC = ldc*N;
            
            TESTING_MALLOC_HOST( h_A,  magmaFloatComplex, lda*An );
            TESTING_MALLOC_HOST( h_B,  magmaFloatComplex, ldb*Bn );
            TESTING_MALLOC_HOST( h_C,  magmaFloatComplex, ldc*N  );
            TESTING_MALLOC_HOST( h_Cmic, magmaFloatComplex, ldc*N  );
            
            TESTING_MALLOC_DEV( d_A, magmaFloatComplex, ldda*An );
            TESTING_MALLOC_DEV( d_B, magmaFloatComplex, lddb*Bn );
            TESTING_MALLOC_DEV( d_C, magmaFloatComplex, lddc*N  );
            
            /* Initialize the matrices */
            lapackf77_clarnv( &ione, ISEED, &sizeA, h_A );
            lapackf77_clarnv( &ione, ISEED, &sizeB, h_B );
            lapackf77_clarnv( &ione, ISEED, &sizeC, h_C );
            
            /* =====================================================================
               Performs operation using MIC-BLAS
               =================================================================== */
            magma_csetmatrix( Am, An, h_A, 0, lda, d_A, 0, ldda, queue );
            magma_csetmatrix( Bm, Bn, h_B, 0, ldb, d_B, 0, lddb, queue );
            magma_csetmatrix( M, N, h_C, 0, ldc, d_C, 0, lddc, queue );
            
            magma_cgemm( opts.transA, opts.transB, M, N, K,
                             alpha, d_A, 0, ldda,
                                    d_B, 0, lddb,
                             beta,  d_C, 0, lddc, queue );
            
            magma_csetmatrix( M, N, h_C, 0, ldc, d_C, 0, lddc, queue );
            
            mic_time = magma_sync_wtime( NULL );
            magma_cgemm( opts.transA, opts.transB, M, N, K,
                         alpha, d_A, 0, ldda,
                                d_B, 0, lddb,
                         beta,  d_C, 0, lddc, queue );
            mic_time = magma_sync_wtime( NULL ) - mic_time;
            mic_perf = gflops / mic_time;
            
            magma_cgetmatrix( M, N, d_C, 0, lddc, h_Cmic, 0, ldc, queue );
            
            /* =====================================================================
               Performs operation using CPU BLAS
               =================================================================== */
            if ( opts.check ) {
                cpu_time = magma_wtime();
                blasf77_cgemm( lapack_const(opts.transA), lapack_const(opts.transB), &M, &N, &K,
                        &alpha, h_A, &lda,
                        h_B, &ldb,
                        &beta,  h_C, &ldc );
                cpu_time = magma_wtime() - cpu_time;
                cpu_perf = gflops / cpu_time;
    
                /* =====================================================================
                   Check the result
                   =================================================================== */
                // compute relative error for both magma & mic, relative to lapack,
                // |C_magma - C_lapack| / |C_lapack|
                Cnorm = lapackf77_clange( "M", &M, &N, h_C, &ldc, work );
    
                blasf77_caxpy( &sizeC, &c_neg_one, h_C, &ione, h_Cmic, &ione );
                mic_error = lapackf77_clange( "M", &M, &N, h_Cmic, &ldc, work ) / Cnorm;
    
                printf("%5d %5d %5d   %7.2f (%7.2f)    %7.2f (%7.2f)    %8.2e\n",
                        (int) M, (int) N, (int) K,
                        mic_perf,  1000.*mic_time,
                        cpu_perf,    1000.*cpu_time,
                        mic_error);
            }
            else {
                printf("%5d %5d %5d   %7.2f (%7.2f)    ---   (  ---  )    ---\n",
                        (int) M, (int) N, (int) K,
                        mic_perf,  1000.*mic_time);
            }
    
            TESTING_FREE_HOST( h_A  );
            TESTING_FREE_HOST( h_B  );
            TESTING_FREE_HOST( h_C  );
            TESTING_FREE_HOST( h_Cmic );
    
            TESTING_FREE_DEV( d_A );
            TESTING_FREE_DEV( d_B );
            TESTING_FREE_DEV( d_C );
        }
        if ( opts.niter > 1 ) {
            printf( "\n" );
        }
    }

    magma_queue_destroy( queue );
    magma_finalize();

    return 0;
}
